@@ -1,6 +1,6 @@
 //
 // tardy - a tar post-processor
-// Copyright (C) 1998, 1999, 2008, 2009 Peter Miller
+// Copyright (C) 1998, 1999, 2008, 2009, 2011 Peter Miller
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,9 +16,14 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 //
 
+#include <libtardy/ac/sys/stat.h>
 #include <libtardy/ac/unistd.h>
+#include <libexplain/fstat.h>
+#include <libexplain/lseek.h>
+#include <libexplain/read.h>
 
 #include <libtardy/file/input/stdin.h>
+#include <libtardy/file/input/position.h>
 
 
 file_input_stdin::~file_input_stdin()
@@ -33,36 +38,50 @@ file_input_stdin::file_input_stdin()
 }
 
 
-file_input_stdin::file_input_stdin(const file_input_stdin &rhs) :
-    file_input(rhs)
+file_input::pointer
+file_input_stdin::create(void)
 {
-    // nothing to do
+    typedef boost::shared_ptr<file_input_stdin> ptr;
+    ptr p(new file_input_stdin());
+    if (p->needs_position_filter())
+        return file_input_position::create(p);
+    return p;
 }
 
 
-file_input_stdin &
-file_input_stdin::operator = (const file_input_stdin &rhs)
+bool
+file_input_stdin::needs_position_filter(void)
+    const
 {
-    file_input::operator=(rhs);
-    // nothing to do
-    return *this;
+    int fd = 0;
+    struct stat st;
+    explain_fstat_or_die(fd, &st);
+    if (!S_ISREG(st.st_mode))
+        return true;
+    return (lseek(fd, 0, SEEK_CUR) != 0);
 }
 
 
 int
-file_input_stdin::read(void *buffer, int maximum_length)
-    const
+file_input_stdin::read_inner(void *data, size_t data_size)
 {
-    int nbytes = ::read(0, buffer, maximum_length);
-    if (nbytes < 0)
-        nfatal("read");
-    return nbytes;
+    int fd = 0;
+    return explain_read_or_die(fd, data, data_size);
 }
 
 
-const char *
-file_input_stdin::filename()
+rcstring
+file_input_stdin::filename(void)
     const
 {
     return "standard input";
 }
+
+
+unsigned long
+file_input_stdin::get_position_inner(void)
+    const
+{
+    int fd = 0;
+    return explain_lseek_or_die(fd, 0, SEEK_CUR);
+}
