@@ -1,6 +1,6 @@
 //
 //      tardy - a tar post-processor
-//      Copyright (C) 1998, 1999, 2003, 2008, 2009 Peter Miller
+//      Copyright (C) 1998, 1999, 2003, 2008, 2009, 2011 Peter Miller
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 //
 
 #include <libtardy/ac/unistd.h>
+#include <libexplain/write.h>
+
 #include <libtardy/file/output/stdout.h>
 
 
@@ -33,19 +35,25 @@ file_output_stdout::file_output_stdout()
 }
 
 
-void
-file_output_stdout::write(const void *buffer, int nbytes)
+file_output::pointer
+file_output_stdout::create(void)
 {
-    int count = ::write(1, buffer, nbytes);
-    if (count < 0)
-        nfatal("write");
-    if (count != nbytes)
-        fatal("short write (gave %d, got %d)", nbytes, count);
+    return pointer(new file_output_stdout());
 }
 
 
-const char *
-file_output_stdout::filename()
+void
+file_output_stdout::write(const void *data, int data_size)
+{
+    int fd = 1;
+    int count = explain_write_or_die(fd, data, data_size);
+    if (count != data_size)
+        fatal("short write (gave %d, got %d)", data_size, count);
+}
+
+
+rcstring
+file_output_stdout::filename(void)
     const
 {
     return "standard output";
