@@ -1,6 +1,6 @@
 //
 //      tardy - a tar post-processor
-//      Copyright (C) 1991-1999, 2001, 2002, 2004, 2008, 2009 Peter Miller
+//      Copyright (C) 1991-1999, 2001, 2002, 2004, 2008, 2009, 2011 Peter Miller
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -22,8 +22,9 @@
 #include <libtardy/ac/stdio.h>
 #include <libtardy/ac/stdlib.h>
 #include <libtardy/ac/string.h>
+#include <libexplain/output.h>
+#include <libexplain/realloc.h>
 
-#include <libtardy/error.h>
 #include <libtardy/mprintf.h>
 #include <libtardy/rcstring.h>
 
@@ -67,21 +68,11 @@ static char     *tmp;
 static int
 bigger(void)
 {
-        char    *hold;
-        size_t  nbytes;
-
-        nbytes = tmplen + QUANTUM;
-        errno = 0;
-        hold = (char *)realloc(tmp, nbytes);
-        if (!hold)
-        {
-                if (!errno)
-                        errno = ENOMEM;
-                return 0;
-        }
-        tmplen = nbytes;
-        tmp = hold;
-        return 1;
+    size_t nbytes = tmplen + QUANTUM;
+    char *hold = (char *)explain_realloc_or_die(tmp, nbytes);
+    tmplen = nbytes;
+    tmp = hold;
+    return 1;
 }
 
 
@@ -658,10 +649,10 @@ mprintf_errok(const char *fmt, ...)
 char *
 vmprintf(const char *fmt, va_list ap)
 {
-        char *result = vmprintf(fmt, ap);
-        if (!result)
-                nfatal("mprintf \"%s\"", fmt);
-        return result;
+    char *result = vmprintf(fmt, ap);
+    if (!result)
+        explain_output_error_and_die("mprintf \"%s\"", fmt);
+    return result;
 }
 
 
@@ -732,7 +723,7 @@ mprintf(const char *fmt, ...)
 string_ty *
 vmprintf_str(const char *fmt, va_list ap)
 {
-        if (!vmprintf_errok(fmt, ap))
-                nfatal("mprintf \"%s\"", fmt);
-        return str_n_from_c(tmp, length);
+    if (!vmprintf_errok(fmt, ap))
+        explain_output_error_and_die("mprintf \"%s\"", fmt);
+    return str_n_from_c(tmp, length);
 }
